/*++

Copyright (c) Sydus.

Module:

	BString.h

Authors:

	Sharvil Nanavati (sharvil) 2005-12-22

--*/

#pragma once

#include "base.h"

class BString
{
	private:
		static const uint32 cDefaultSize = 256;

	public:
		//
		// Compatibility with std C++
		//
		typedef uint8 value_type;
		typedef size_t size_type;
		typedef size_t difference_type;
		typedef value_type * pointer;
		typedef value_type & reference;
		typedef const value_type * const_pointer;
		typedef const value_type & const_reference;

		static const size_type npos = static_cast <size_type> (-1);

		bool empty() const;
		size_type size() const;
		size_type length() const;
		size_type max_size() const;
		size_type capacity() const;

		void clear();

		//
		// BString members
		//
		BString();
		explicit BString(size_type size);
		explicit BString(const std::string & str);
		BString(const BString & copy);
		BString(const void * data, size_type size);
		~BString();

		void Append(value_type datum);
		void Append(const void * data, size_type size);
		void Append(value_type value, size_type count);
		void Append(const BString & str);
		void Assign(const void * data, size_type size);
		void Fill(value_type value, size_type offset, size_type count);
		size_type Find(const BString & value, size_type offset = 0) const;
		size_type Find(const std::string & value, size_type offset = 0) const;
		size_type Find(const void * value, size_type count, size_type offset = 0) const;
		void Resize(size_type newSize);
		void Reserve(size_type size);
		BString Mid(size_type offset, size_type length = npos) const;
		BString SubString(size_type offset = 0, size_type count = npos) const;
		void Zero();

		uint8 * Data();
		const uint8 * Data() const;

		bool IsEmpty() const;
		size_type GetSize() const;
		size_type GetLength() const;
		size_type GetCapacity() const;

		reference operator [] (size_type index);
		const_reference operator [] (size_type index) const;

		BString & operator = (const BString & rhs);
		BString & operator = (const std::string & rhs);

		std::string AsString() const;

	private:
		uint8 * iData;
		size_type iSize;
		size_type iCapacity;
};

bool operator == (const BString & lhs, const BString & rhs);
bool operator != (const BString & lhs, const BString & rhs);
bool operator <  (const BString & lhs, const BString & rhs);
bool operator >  (const BString & lhs, const BString & rhs);
bool operator <= (const BString & lhs, const BString & rhs);
bool operator >= (const BString & lhs, const BString & rhs);
